@@ -1,12 +1,10 @@
 ;;; helm-system-packages-dpkg.el --- Helm UI for Debian's dpkg. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2014 Thierry Volpiatto <thierry.volpiatto@gmail.com>
-;;               2017 ~ 2018 Pierre Neidhardt <mail@ambrevar.xyz>
+;; Copyright (C) 2017 ~ 2020 Pierre Neidhardt <mail@ambrevar.xyz>
+;;               2020 ~ 2025 Thierry Volpiatto <thievol@posteo.net>
 
 ;; Author: Pierre Neidhardt <mail@ambrevar.xyz>
 ;; URL: https://github.com/emacs-helm/helm-system-packages
-;; Package-Requires: ((emacs "24.4") (helm "2.8.6"))
-;; Keywords: helm, packages
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -52,11 +50,11 @@ Requirements:
   ;; M-U is reserved for `helm-unmark-all'.
   (let ((map (make-sparse-keymap)))
     (set-keymap-parent map helm-map)
-    (define-key map (kbd "M-I")   'helm-system-packages-dpkg-toggle-explicit)
-    (define-key map (kbd "M-N")   'helm-system-packages-dpkg-toggle-uninstalled)
-    (define-key map (kbd "M-D")   'helm-system-packages-dpkg-toggle-dependencies)
-    (define-key map (kbd "M-R")   'helm-system-packages-dpkg-toggle-residuals)
-    (define-key map (kbd "C-]")   'helm-system-packages-toggle-descriptions)
+    (define-key map (kbd "M-I") 'helm-system-packages-dpkg-toggle-explicit)
+    (define-key map (kbd "M-N") 'helm-system-packages-dpkg-toggle-uninstalled)
+    (define-key map (kbd "M-D") 'helm-system-packages-dpkg-toggle-dependencies)
+    (define-key map (kbd "M-R") 'helm-system-packages-dpkg-toggle-residuals)
+    (define-key map (kbd "C-]") 'helm-system-packages-toggle-descriptions)
     map))
 
 (defvar helm-system-packages-dpkg--show-uninstalled-p t)
@@ -64,55 +62,42 @@ Requirements:
 (defvar helm-system-packages-dpkg--show-dependencies-p t)
 (defvar helm-system-packages-dpkg--show-residuals-p t)
 
-(defun helm-system-packages-dpkg-toggle-explicit ()
-  (interactive)
-  (with-helm-alive-p
-    (setq helm-system-packages-dpkg--show-explicit-p
-          (not helm-system-packages-dpkg--show-explicit-p))
-    (helm-update)))
-(put 'helm-system-packages-dpkg-toggle-explicit 'helm-only t)
+(defcustom helm-system-packages-dpkg-actions
+  '(("Show package(s)" . helm-system-packages-dpkg-info)
+    ("Install (`C-u' to reinstall)" .
+     (lambda (_)
+       (apply 'helm-system-packages-run-as-root
+              (helm-system-packages-make-apt-get-command
+               "install"
+               (and helm-current-prefix-arg "--reinstall")))))
+    ("Uninstall (`C-u' to include dependencies)" .
+     (lambda (_)
+       (apply 'helm-system-packages-run-as-root-over-installed
+              (helm-system-packages-make-apt-get-command
+               "remove"
+               (and helm-current-prefix-arg "--auto-remove")))))
+    ("Browse homepage URL" . helm-system-packages-dpkg-browse-url)
+    ("Find files" . helm-system-packages-dpkg-find-files)
+    ("Show dependencies" . helm-system-packages-dpkg-show-dependencies)
+    ("Show reverse dependencies" .
+     (lambda (_)
+       (helm-system-packages-dpkg-show-dependencies _ 'reverse)))
+    ("Uninstall/Purge (`C-u' to include dependencies)" .
+     (lambda (_)
+       (apply 'helm-system-packages-run-as-root-over-installed
+              (helm-system-packages-make-apt-get-command
+               "purge"
+               (and helm-current-prefix-arg "--auto-remove"))))))
+  "Actions for Helm dpkg."
+  :group 'helm-system-packages
+  :type '(alist :key-type string :value-type function))
 
-(defun helm-system-packages-dpkg-toggle-uninstalled ()
-  (interactive)
-  (with-helm-alive-p
-    (setq helm-system-packages-dpkg--show-uninstalled-p
-          (not helm-system-packages-dpkg--show-uninstalled-p))
-    (helm-update)))
-(put 'helm-system-packages-dpkg-toggle-uninstalled 'helm-only t)
-
-(defun helm-system-packages-dpkg-toggle-dependencies ()
-  (interactive)
-  (with-helm-alive-p
-    (setq helm-system-packages-dpkg--show-dependencies-p
-          (not helm-system-packages-dpkg--show-dependencies-p))
-    (helm-update)))
-(put 'helm-system-packages-dpkg-toggle-dependencies 'helm-only t)
-
-(defun helm-system-packages-dpkg-toggle-residuals ()
-  (interactive)
-  (with-helm-alive-p
-    (setq helm-system-packages-dpkg--show-residuals-p
-          (not helm-system-packages-dpkg--show-residuals-p))
-    (helm-update)))
-(put 'helm-system-packages-dpkg-toggle-residuals 'helm-only t)
-
-(defun helm-system-packages-dpkg-transformer (packages)
-  (let (res (pkglist (reverse packages)))
-    (dolist (p pkglist res)
-      (let ((face (cdr (assoc (helm-system-packages-extract-name p)
-                              (plist-get (helm-system-packages--cache-get) :display)))))
-        (cond
-         ((not face) (when helm-system-packages-dpkg--show-uninstalled-p
-                       (push p res)))
-         ((or
-           (and helm-system-packages-dpkg--show-explicit-p
-                (memq 'helm-system-packages-explicit face))
-           (and helm-system-packages-dpkg--show-dependencies-p
-                (memq 'helm-system-packages-dependencies face))
-           (and helm-system-packages-dpkg--show-residuals-p
-                (memq 'helm-system-packages-residuals face)))
-          (push (propertize p 'face (car face)) res)))))))
+(defcustom helm-system-packages-dpkg-column-width 40
+  "Column at which descriptions are aligned, excluding a double-space gap."
+  :group 'helm-system-packages
+  :type 'integer)
 
+;; Functions for caching and filtering.
 (defun helm-system-packages-dpkg-list-explicit ()
   "List explicitly installed packages."
   (split-string (with-temp-buffer
@@ -139,35 +124,27 @@ Requirements:
         (when (string= (cadr pkg) "deinstall")
           (push (car pkg) res))))))
 
-(defun helm-system-packages-dpkg-cache-names () ; TODO: Build from --descriptions instead like pacman?
-  "Cache all package names."
-  (with-temp-buffer
-    (process-file "apt-cache" nil t nil "pkgnames")
-    ;; (sort-lines nil (point-min) (point-max))
-    (buffer-string)))
-
-(defcustom helm-system-packages-dpkg-column-width 40
-  "Column at which descriptions are aligned, excluding a double-space gap."
-  :group 'helm-system-packages
-  :type 'integer)
-
-(defun helm-system-packages-dpkg-cache-descriptions ()
+(defun helm-system-packages-dpkg-cache (display-list)
   "Cache all package names with descriptions."
-  (with-temp-buffer
-    ;; `apt-cache search` is much faster than `apt-cache show`.
-    (process-file "apt-cache" nil '(t nil) nil "search" ".")
-    ;; apt-cache's output format is "pkg - desc".  Remove "-" and align to
-    ;; column.
-    (goto-char (point-min))
-    (while (search-forward " " nil t)
-      (delete-char 1)
-      (backward-char)
-      (let ((pos (- (point) (line-beginning-position))))
-        (when (< pos helm-system-packages-dpkg-column-width)
-          (insert (make-string (- helm-system-packages-dpkg-column-width pos) ? ))))
-      (forward-line))
-    ;; (sort-lines nil (point-min) (point-max)) ; TODO: Required? Also see helm-system-packages-dpkg-buffer-all.
-    (buffer-string)))
+  (let (names descriptions)
+    (with-temp-buffer
+      ;; `apt-cache search` is much faster than `apt-cache show`.
+      (process-file "apt-cache" nil '(t nil) nil "search" ".")
+      (goto-char (point-min))
+      (while (re-search-forward "^\\([^ ]*\\)\\( - \\)" nil t)
+        (let ((name (match-string 1))
+              (pos  (- (match-end 1) (pos-bol))))
+          (push name names)
+          ;; apt-cache's output format is "pkg - desc".
+          ;; Remove " - " and align to column.
+          (when (< pos helm-system-packages-dpkg-column-width)
+            (replace-match (make-string
+                            (- helm-system-packages-dpkg-column-width pos) ? )
+                           nil nil nil 2)))
+        (forward-line))
+      (setq descriptions (buffer-string)))
+    (setq names (mapconcat #'identity names "\n"))
+    (helm-system-packages--cache-set names descriptions display-list "dpkg")))
 
 (defun helm-system-packages-dpkg-refresh ()
   "Refresh the package list."
@@ -185,10 +162,46 @@ Requirements:
     (dolist (p residuals)
       (push (cons p '(helm-system-packages-residuals))
             display-list))
-    (helm-system-packages--cache-set
-     (helm-system-packages-dpkg-cache-names)
-     (helm-system-packages-dpkg-cache-descriptions)
-     display-list "dpkg")))
+    (helm-system-packages-dpkg-cache display-list)))
+
+(defun helm-system-packages-dpkg-transformer (packages _source)
+  (let (res
+        (disps (plist-get (helm-system-packages--cache-get)
+                          :filtered)))
+    (dolist (p packages)
+      (let* ((name (helm-system-packages-extract-name p))
+             (face (cdr (assoc name disps))))
+        (cond
+         ((not face)
+          (when helm-system-packages-dpkg--show-uninstalled-p
+            (push p res)))
+         ((or
+           (and helm-system-packages-dpkg--show-explicit-p
+                (memq 'helm-system-packages-explicit face))
+           (and helm-system-packages-dpkg--show-dependencies-p
+                (memq 'helm-system-packages-dependencies face))
+           (and helm-system-packages-dpkg--show-residuals-p
+                (memq 'helm-system-packages-residuals face)))
+          (push (propertize p 'face (car face)) res)))))
+    (nreverse res)))
+
+
+;; Actions
+(defun helm-system-packages-dpkg-toggle-explicit ()
+  (interactive)
+  (with-helm-alive-p
+    (setq helm-system-packages-dpkg--show-explicit-p
+          (not helm-system-packages-dpkg--show-explicit-p))
+    (helm-update)))
+(put 'helm-system-packages-dpkg-toggle-explicit 'helm-only t)
+
+(defun helm-system-packages-dpkg-toggle-uninstalled ()
+  (interactive)
+  (with-helm-alive-p
+    (setq helm-system-packages-dpkg--show-uninstalled-p
+          (not helm-system-packages-dpkg--show-uninstalled-p))
+    (helm-update)))
+(put 'helm-system-packages-dpkg-toggle-uninstalled 'helm-only t)
 
 (defun helm-system-packages-dpkg-info (candidate)
   "Print information about the selected packages.
@@ -198,7 +211,8 @@ Otherwise display in `helm-system-packages-buffer'."
    ;; TODO: Optimize by calling the command only once and parsing output.
    `((uninstalled . ,(mapcar (lambda (pkg)
                                (cons pkg
-                                     (helm-system-packages-call "apt-cache" nil "show" pkg)))
+                                     (helm-system-packages-call
+                                      "apt-cache" nil "show" pkg)))
                              (if helm-in-persistent-action
                                  (list candidate)
                                (helm-marked-candidates)))))))
@@ -208,7 +222,8 @@ Otherwise display in `helm-system-packages-buffer'."
 
 With prefix argument, insert the output at point.
 Otherwise display in `helm-system-packages-buffer'."
-  (let ((res (helm-system-packages-call "apt-cache" (helm-marked-candidates) "show"))
+  (let ((res (helm-system-packages-call
+              "apt-cache" (helm-marked-candidates) "show"))
         urls)
     (dolist (url (split-string res "\n" t))
       (when (string-match "^Homepage: \\(.*\\)" url)
@@ -226,7 +241,24 @@ Otherwise display in `helm-system-packages-buffer'."
                       "\n" t))
          (push file (gethash pkg file-hash)))))))
 
-(defun helm-system-packages-dpkg-show-dependencies (_candidate &optional reverse)
+(defun helm-system-packages-dpkg-toggle-dependencies ()
+  (interactive)
+  (with-helm-alive-p
+    (setq helm-system-packages-dpkg--show-dependencies-p
+          (not helm-system-packages-dpkg--show-dependencies-p))
+    (helm-update)))
+(put 'helm-system-packages-dpkg-toggle-dependencies 'helm-only t)
+
+(defun helm-system-packages-dpkg-toggle-residuals ()
+  (interactive)
+  (with-helm-alive-p
+    (setq helm-system-packages-dpkg--show-residuals-p
+          (not helm-system-packages-dpkg--show-residuals-p))
+    (helm-update)))
+(put 'helm-system-packages-dpkg-toggle-residuals 'helm-only t)
+
+(defun helm-system-packages-dpkg-show-dependencies (_candidate
+                                                    &optional reverse)
   "List candidate dependencies for `helm-system-packages-show-packages'.
 If REVERSE is non-nil, list reverse dependencies instead."
   (let ((arg (if reverse "rdepends" "depends"))
@@ -239,40 +271,14 @@ If REVERSE is non-nil, list reverse dependencies instead."
                         "apt-cache" (helm-marked-candidates) arg)))
      title)))
 
-(defun helm-system-packages-make-apt-get-command ( &rest args)
+(defun helm-system-packages-make-apt-get-command (&rest args)
   (let ((comm (append '("apt-get" "--quiet") args)))
     (if helm-system-packages-dpkg-confirm-p
         (push "DEBIAN_FRONTEND=readline" comm)
       (push "DEBIAN_FRONTEND=noninteractive" comm))))
 
-(defcustom helm-system-packages-dpkg-actions
-  '(("Show package(s)" . helm-system-packages-dpkg-info)
-    ("Install (`C-u' to reinstall)" .
-     (lambda (_)
-       (apply 'helm-system-packages-run-as-root
-              (helm-system-packages-make-apt-get-command "install"
-                                                         (when helm-current-prefix-arg "--reinstall")))))
-    ("Uninstall (`C-u' to include dependencies)" .
-     (lambda (_)
-       (apply 'helm-system-packages-run-as-root-over-installed
-              (helm-system-packages-make-apt-get-command "remove"
-                                                         (when helm-current-prefix-arg "--auto-remove")))))
-    ("Browse homepage URL" . helm-system-packages-dpkg-browse-url)
-    ("Find files" . helm-system-packages-dpkg-find-files)
-    ("Show dependencies" . helm-system-packages-dpkg-show-dependencies)
-    ("Show reverse dependencies" .
-     (lambda (_)
-       (helm-system-packages-dpkg-show-dependencies _ 'reverse)))
-    ("Uninstall/Purge (`C-u' to include dependencies)" .
-     (lambda (_)
-       (apply 'helm-system-packages-run-as-root-over-installed
-              (helm-system-packages-make-apt-get-command "purge"
-                                                         (when helm-current-prefix-arg "--auto-remove"))))))
-  "Actions for Helm dpkg."
-  :group 'helm-system-packages
-  :type '(alist :key-type string :value-type function))
-
-(defvar helm-system-packages-dpkg-dependencies '("apt-get" "apt-cache" "apt-mark" "dpkg"))
+(defvar helm-system-packages-dpkg-dependencies
+  '("apt-get" "apt-cache" "apt-mark" "dpkg"))
 (defvar helm-system-packages-dpkg
   (helm-system-packages-manager-create
    :name "dpkg"
@@ -280,7 +286,7 @@ If REVERSE is non-nil, list reverse dependencies instead."
    :dependencies helm-system-packages-dpkg-dependencies
    :help-message 'helm-system-packages-dpkg-help-message
    :keymap helm-system-packages-dpkg-map
-   :transformer #'helm-system-packages-dpkg-transformer
+   :fc-transformer #'helm-system-packages-dpkg-transformer
    :actions helm-system-packages-dpkg-actions))
 
 (provide 'helm-system-packages-dpkg)
