@@ -1,12 +1,10 @@
 ;;; helm-system-packages-guix.el --- Helm UI for Guix. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2014 Thierry Volpiatto <thierry.volpiatto@gmail.com>
-;;               2017 ~ 2018, 2020 Pierre Neidhardt <mail@ambrevar.xyz>
+;; Copyright (C) 2017 ~ 2020 Pierre Neidhardt <mail@ambrevar.xyz>
+;;               2020 ~ 2025 Thierry Volpiatto <thievol@posteo.net>
 
 ;; Author: Pierre Neidhardt <mail@ambrevar.xyz>
 ;; URL: https://github.com/emacs-helm/helm-system-packages
-;; Package-Requires: ((emacs "25") (helm "2.8.6"))
-;; Keywords: helm, guix, packages
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -82,7 +80,7 @@ Requirements:
   (let (res (pkglist (reverse packages)))
     (dolist (p pkglist res)
       (let ((face (cdr (assoc (helm-system-packages-extract-name p)
-                              (plist-get (helm-system-packages--cache-get) :display)))))
+                              (plist-get (helm-system-packages--cache-get) :filtered)))))
         (cond
          ((not face)
           (when helm-system-packages-guix--show-uninstalled-p
