@@ -1,12 +1,10 @@
 ;;; helm-system-packages-pacman.el --- Helm UI for Arch Linux' pacman. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2014 Thierry Volpiatto <thierry.volpiatto@gmail.com>
-;;               2017 ~ 2018 Pierre Neidhardt <mail@ambrevar.xyz>
+;; Copyright (C) 2017 ~ 2020 Pierre Neidhardt <mail@ambrevar.xyz>
+;;               2020 ~ 2025 Thierry Volpiatto <thievol@posteo.net>
 
 ;; Author: Pierre Neidhardt <mail@ambrevar.xyz>
 ;; URL: https://github.com/emacs-helm/helm-system-packages
-;; Package-Requires: ((emacs "24.4") (helm "2.8.6"))
-;; Keywords: helm, packages
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -101,7 +99,8 @@ tested package to fall back on."
     ("Browse homepage URL" .
      (lambda (_)
        (helm-system-packages-browse-url
-        (split-string (helm-system-packages-run "expac" "--sync" "%u")
+        (split-string (helm-system-packages-call
+                       "expac" (helm-marked-candidates) "--sync" "%u")
                       "\n" t))))
     ("Find files" . helm-system-packages-pacman-find-files)
     ("Show dependencies (`C-u' to include optional deps)" .
@@ -183,53 +182,48 @@ packages belonging to the group."
       (push 'helm-system-packages-locals (cdr (assoc p display-list))))
     (dolist (p groups)
       (push (cons p '(helm-system-packages-groups)) display-list))
-    (helm-system-packages-pacman-cache display-list locals groups)))
+    (helm-system-packages-pacman-cache display-list groups)))
 
-(defun helm-system-packages-pacman-cache (display-list local-packages groups)
+(defun helm-system-packages-pacman-cache (display-list groups)
   "Cache all package names with descriptions.
-LOCAL-PACKAGES and GROUPS are lists of strings."
-  ;; We build both caches at the same time.  We could also build
-  ;; just-in-time, but benchmarks show that it only saves less than
-  ;; 20% when building one cache.
+DISPLAY-LIST and GROUPS are lists of strings."
   (let (names descriptions)
-    (setq descriptions
-          (with-temp-buffer
-            ;; TODO: Possible optimization: Output directly in Elisp?
-            (let ((format-string (format "%%-%dn  %%d"
-                                         helm-system-packages-column-width)))
-              (process-file "expac" nil '(t nil) nil "--sync" format-string)
-              (apply 'process-file "expac" nil '(t nil) nil
-                     "--query" format-string local-packages))
-            (dolist (g groups)
-              (insert (concat g
-                              (make-string (- helm-system-packages-column-width
-                                              (length g))
-                                           ? )
-                              "  <group>\n")))
-            (buffer-string)))
-    (setq names (mapconcat #'car display-list "\n"))
+    (with-temp-buffer
+      (let ((format-string (format "%%-%dn  %%d"
+                                   helm-system-packages-column-width)))
+        (process-file "expac" nil '(t nil) nil "--sync" format-string))
+      (dolist (g groups)
+        (insert (concat g
+                        (make-string (- helm-system-packages-column-width
+                                        (length g))
+                                     ? )
+                        "  <group>\n")))
+      (goto-char (point-min))
+      (while (re-search-forward "^\\([^ ]*\\) +" nil t)
+        (push (match-string 1) names))
+      (setq descriptions (buffer-string)))
+    (setq names (mapconcat #'identity names "\n"))
     (helm-system-packages--cache-set names descriptions display-list "pacman")))
 
 (defun helm-system-packages-pacman-transformer (packages)
-  (let ((res '()))
-    (dolist (p (sort (copy-sequence packages)))
-      (let ((face (cdr (assoc (helm-system-packages-extract-name p)
-                              (plist-get (helm-system-packages--cache-get)
-                                         :display)))))
+  (let ((res '())
+        (disps (plist-get (helm-system-packages--cache-get) :filtered)))
+    (dolist (p (sort packages #'string-lessp))
+      (let* ((name (helm-system-packages-extract-name p))
+             (len  (length name))
+             (face (cdr (assoc name disps))))
         (cond
-         ((and (not face) (member (helm-system-packages-extract-name p)
-                                  helm-system-packages--virtual-list))
+         ((and (not face)
+               (member name helm-system-packages--virtual-list))
           ;; When displaying dependencies, package may be virtual.
           ;; Check first since it is also an "uninstalled" package.
-          (push (propertize p 'face 'helm-system-packages-pacman-virtual) res))
-         ((and (not face) helm-system-packages--show-uninstalled-p)
-          (push p res))
+          (add-face-text-property 0 len
+                                  'helm-system-packages-pacman-virtual nil p))
          ;; For filtering, we consider local packages and non-local packages
          ;; separately, thus we need to treat local packages first.
-         ;; TODO: Add support for multiple faces.
          ((memq 'helm-system-packages-locals face)
           (when helm-system-packages--show-locals-p
-            (push (propertize p 'face (car face)) res)))
+            (add-face-text-property 0 len (car face) nil p)))
          ((or
            (and helm-system-packages--show-explicit-p
                 (memq 'helm-system-packages-explicit face))
@@ -239,8 +233,11 @@ LOCAL-PACKAGES and GROUPS are lists of strings."
                 (memq 'helm-system-packages-orphans face))
            (and helm-system-packages--show-groups-p
                 (memq 'helm-system-packages-groups face)))
-          (push (propertize p 'face (car face)) res)))))
-    (helm-fast-remove-dups (reverse res) :test #'equal)))
+          (add-face-text-property 0 len (car face) nil p)))
+        (add-face-text-property
+         len (length p) 'helm-system-packages-descriptions nil p)
+        (push p res)))
+    (helm-fast-remove-dups (nreverse res) :test #'equal)))
 
 ;; Actions
 (defun helm-system-packages-pacman-outdated-database-p ()
