@@ -1,13 +1,11 @@
 ;;; helm-system-packages-portage.el --- Helm UI for Portage. -*- lexical-binding: t -*-
 
-;; Copyright (C) 2012 ~ 2014 Thierry Volpiatto <thierry.volpiatto@gmail.com>
-;;               2017 ~ 2018 Pierre Neidhardt <mail@ambrevar.xyz>
+;; Copyright (C) 2017 ~ 2020 Pierre Neidhardt <mail@ambrevar.xyz>
+;;               2020 ~ 2025 Thierry Volpiatto <thievol@posteo.net>
 
-;; Author: Thierry Volpiatto <thierry.volpiatto@gmail.com>
+;; Author: Thierry Volpiatto <thievol@posteo.com>
 ;; Maintainer: Pierre Neidhardt <mail@ambrevar.xyz>
 ;; URL: https://github.com/emacs-helm/helm-system-packages
-;; Package-Requires: ((emacs "24.4") (helm "2.8.6"))
-;; Keywords: helm, packages
 
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -56,7 +54,7 @@
   (let (res (pkglist (reverse packages)))
     (dolist (p pkglist res)
       (let ((face (cdr (assoc (helm-system-packages-extract-name p)
-                              (plist-get (helm-system-packages--cache-get) :display)))))
+                              (plist-get (helm-system-packages--cache-get) :filtered)))))
         (cond
          ((and (not face) helm-system-packages--show-uninstalled-p)
           (push p res))
